import socket
import sys
import re
import logging

from ethz import Error
from ethz import util
from ethz.api import komcenter



class NetworkError(Error):
    """Base exception for this module"""
    pass



_komcenter = None
def get_komcenter_client():
    global _komcenter
    if not _komcenter:
        username,password = util.get_username_password(username=None, host=komcenter.hostname)
        _komcenter = komcenter.Client(username=username, password=password)
    return _komcenter


mac_re = re.compile(r'^(?:[a-f0-9]{2}[\:-]){5}[a-f0-9]{2}$', re.IGNORECASE)
def is_mac_address(mac):
    if mac_re.search(mac):
        return True
    else:
        return False


def get_host_info(ip_name_mac_cable, komcenter_client=None):
    kc = komcenter_client or get_komcenter_client()
    hosts = list(kc.get_host_info(ip_name_mac_cable))
    if not hosts:
        raise komcenter.KomcenterError('Failed to get host info for %s' % ip_name_mac_cable)
    return hosts[0]


def get_mac(hostname_or_ip, komcenter_client=None):
    try:
        host = get_host_info(hostname_or_ip, komcenter_client=komcenter_client)
        if host:
            return host['mac']
    except komcenter.KomcenterAuthenticationError:
        raise
    except komcenter.KomcenterError, e:
        logging.warning(e)
    raise NetworkError('Failed to get MAC address for \'%s\' from komcenter.' % hostname_or_ip)


def get_ip(hostname_or_mac, komcenter_client=None):
    try:
        host = get_host_info(hostname_or_mac, komcenter_client=komcenter_client)
        if host:
            return host['ip']
    except komcenter.KomcenterAuthenticationError:
        raise
    except komcenter.KomcenterError, e:
        logging.warning(e)

    if is_mac_address(hostname_or_mac):
        raise NetworkError('Failed to get IP for MAC \'%s\' from komcenter.' % hostname_or_mac)
    else:
        try:
            # Check the DNS database
            kc = komcenter_client or get_komcenter_client()
            ip = kc.get_host_ip(hostname_or_mac)
            if ip:
                return ip
        except komcenter.KomcenterAuthenticationError:
            raise
        except komcenter.KomcenterError, e:
            logging.warning(e)
        
        logging.warning('Failed to get IP for hostname \'%s\' from komcenter. Falling back to socket module.', hostname_or_mac)
        return get_ip_from_hostname(hostname_or_mac)


def get_hostname(ip_or_mac, komcenter_client=None):
    try:
        host = get_host_info(ip_or_mac, komcenter_client=komcenter_client)
        if host:
            return host['ip_name']
    except komcenter.KomcenterAuthenticationError:
        raise
    except komcenter.KomcenterError, e:
        logging.warning(e)

    if is_mac_address(ip_or_mac):
        raise NetworkError('Failed to get hostname for MAC \'%s\' from komcenter.' % ip_or_mac)
    else:
        logging.warning('Failed to get hostname for IP \'%s\' from komcenter. Falling back to socket module.', ip_or_mac)
        return get_hostname_from_ip(ip_or_mac)


def get_ip_from_hostname(hostname):
    try:
        return socket.gethostbyname(hostname)
    except socket.gaierror, (code, message):
        raise NetworkError('Failed to get IP for hostname \'%s\': %s, %s' % (hostname, code, message))


def get_hostname_from_ip(ip):
    try:
        return socket.gethostbyaddr(ip)[0]
    except socket.gaierror, (code, message):
        raise NetworkError('Failed to get hostname for IP \'%s\': %s, %s' % (ip, code, message))
    except IndexError, e:
        raise NetworkError('Failed to get hostname for IP \'%s\': %s' % (ip, e))



